<?php

namespace Drupal\vipps_recurring_payments_commerce\PluginForm\OffsiteRedirect;


use Drupal;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\vipps_recurring_payments\Entity\VippsProductSubscription;
use Drupal\vipps_recurring_payments\Form\SettingsForm;
use Drupal\vipps_recurring_payments\Service\VippsHttpClient;
use Drupal\vipps_recurring_payments\Service\VippsService;
use Exception;
use Symfony\Component\DependencyInjection\ContainerInterface;

class VippsPaymentGatewayCheckout extends BasePaymentOffsiteForm implements ContainerInjectionInterface {


  /**
   * @var \Drupal\vipps_recurring_payments\Service\VippsHttpClient
   */
  protected $httpClient;

  /**
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * @var \Drupal\vipps_recurring_payments\Service\VippsService
   */
  protected $vippsService;

  /**
   * BaseVippsPaymentGatewayForm constructor.
   *
   * @param VippsHttpClient        $httpClient
   * @param ConfigFactoryInterface $configFactory
   * @param VippsService           $vippsService
   */
  public function __construct(VippsHttpClient $httpClient, ConfigFactoryInterface $configFactory, VippsService $vippsService) {
    $this->httpClient = $httpClient;
    $this->configFactory = $configFactory;
    $this->vippsService = $vippsService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('vipps_recurring_payments:http_client'),
      $container->get('config.factory'),
      $container->get('vipps_recurring_payments:vipps_service')
    );
  }

  /**
   * {@inheritdoc}
   * @throws \Drupal\commerce\Response\NeedsRedirectException
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\vipps_recurring_payments\Factory\RequestStorageFactory $requestStorageFactory */
    $requestStorageFactory = Drupal::service('vipps_recurring_payments:request_storage_factory');


    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;

    /** @var  \Drupal\commerce_order\Entity\Order $order */
    $order = $form_state->getFormObject()->getOrder();

    /** @var SettingsForm $settings */
    $configFactory = $this->configFactory->getEditable(SettingsForm::SETTINGS);
    $settings = $configFactory->getRawData();

    /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    $payment_gateway_settings = $payment_gateway_plugin->getConfiguration();

    if($settings['msn'] == '' || $settings['test_msn'] == '') {
      Drupal::logger('vipps_recurring_commerce')->error(
        'There is no Settings for Vipps recurring'
      );
      throw new PaymentGatewayException('There is no Settings for Vipps recurring');
    }


    $token = $this->httpClient->auth();
    $order->setOrderNumber($order->id());
    $order->save();
    $title = ' ';

    // Can be considered an initial subscription order if it has at least one
    // product which has subscription enabled.
    foreach ($order->getItems() as $order_item) {
      $purchased_entity = $order_item->getPurchasedEntity();
      $title = $purchased_entity->getTitle();
    }

    if ($order->getData('vipps_auth_key') === NULL) {
      $order->setData('vipps_auth_key', $token);
    }

    /** @var  $intervalService */
    $intervalService = Drupal::service('vipps_recurring_payments:charge_intervals');
    $intervals = $intervalService->getIntervals($payment_gateway_settings['charge_interval']);

    $product = new VippsProductSubscription(
      $intervals['base_interval'],
      intval($intervals['base_interval_count']),
      $title,
      $title,
      $payment_gateway_settings['initial_charge']
    );
    $product->setPrice($order->getTotalPrice()->getNumber());

    try {
      $draftAgreementResponse = $this->httpClient->draftAgreement(
        $token,
        $requestStorageFactory->buildDefaultDraftAgreement(
          $product,
          '',
          [
            'commerce_order' => $order->id(),
            'step' => 'payment'
          ]
        )
      );
    }
    catch (Exception $exception) {
      Drupal::logger('vipps_recurring_commerce')->error(
        'Order %oid: problems drafting the agreement', ['%oid' => $order->id()]
      );
      throw new PaymentGatewayException($exception->getMessage());
    }

    $payment->setRemoteId($draftAgreementResponse->getAgreementId());
    $payment->save();

    $order->setData('vipps_current_transaction', $draftAgreementResponse->getAgreementId());
    $order->save();

    // Builds the redirect form.
    return $this->buildRedirectForm($form, $form_state, $draftAgreementResponse->getVippsConfirmationUrl(), []);
  }


}
